public class Hanoi {

    private final Peg peg1, peg2, peg3;

    private int largestDisk;

    private int steps;

    // Create a new Hanoi puzzle with the given number of disks on the left peg.
    Hanoi(int nbOfDisks) {
        peg1 = new Peg(Peg.Position.LEFT);
        peg2 = new Peg(Peg.Position.CENTER);
        peg3 = new Peg(Peg.Position.RIGHT);

        largestDisk = nbOfDisks;
        steps = 0;

        for (int size = nbOfDisks; size > 0; size--) {
            peg1.addDisk(size);
        }
    }

    // Move one disk from 'source' to 'destination'
    // @requires: 'source' has at least one disk
    // @requires: the top disk at 'destination' (if any) is bigger than
    //            the top
    public void moveOneDisk(Peg source, Peg destination) {
        System.out.println(
                "Moving disk from " + source.getPosition() +
                        " to " + destination.getPosition()
        );
        int disk = source.removeDisk();
        destination.addDisk(disk);
        steps++;
        System.out.println(this);
    }

    /* Move 'n' top disks from 'source' peg to 'destination' peg via 'spare' peg
       @requires: 'source' has at least 'n' disks
       @requires: top disk at 'destination' and 'spare' (if any) are bigger than
                  the 'n'th disk at 'source'
       @ensures:  top 'n' disks of 'source' have been moved to 'destination'
    */
    public void moveDisks(int n,
        Peg source, Peg destination, Peg spare) {

        if (n == 0) {
            return;
        } else if (n == 1) {
            moveOneDisk(source, destination);
        } else {
            moveDisks(n-1,source,spare,destination);
            moveOneDisk(source,destination);
            moveDisks(n-1,spare,destination,source);
        }


    }

    public String toString() {
        return  peg1.toString() + "\n" +
                peg2.toString() + "\n" +
                peg3.toString();
    }

    public static void main(String[] args) {

        int disks = 7;

        Hanoi puzzle = new Hanoi(disks);
        puzzle.moveDisks(disks,puzzle.peg1,puzzle.peg2,puzzle.peg3);
        System.out.println("Total number of steps: " + puzzle.steps);
    }

}


























        /*
        if (n == 1)
            // base case
            moveOneDisk(source, destination);
        else {
            // recursively move n - 1 to spare
            moveDisks(n - 1, source, spare, destination);
            // move largest disk to destination
            moveOneDisk(source, destination);
            // recursively move n - 1 to destination
            moveDisks(n - 1, spare, destination, source);
        }
        */