import java.util.ArrayList;
import java.util.List;

public class Peg {

    public enum Position {
        LEFT("Left"), CENTER("Center"), RIGHT("Right");

        private String name;

        Position(String name) {
            this.name = name;
        }

        @Override
        public String toString() {
            return name;
        }

    }

    private final Position pos;

    // @invariant: disks are ordered from small to big
    private List<Integer> disks;

    // Create a new peg with no disks at the given position
    public Peg(Position pos) {
        this.pos = pos;
        disks = new ArrayList<>();
    }

    // Get the position of this peg
    public Position getPosition() {
        return pos;
    }

    // Add a disk to this peg
    // @requires: either this peg is empty or the top-most disk is bigger than the new disk
    public void addDisk(int disk) {
        assert disks.isEmpty() || disk < disks.get(0);
        disks.add(0,disk);
    }

    // Removes the top-most disk from this peg and return its size
    // @requires: there is at least one disk on this peg
    public int removeDisk() {
        assert !disks.isEmpty();
        int disk = disks.get(0);
        disks.remove(0);
        return disk;
    }

    @Override
    public String toString() {
        return pos.toString() + " peg: " + disks.toString();
    }

}
